/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

import org.objectweb.dream.util.EmptyIterator;
import org.objectweb.dream.util.EmptyStringArray;

/**
 * Abstract implementation of the <code>ExtensibleMessage</code> interface.
 * Methods managing sub messages are not implemented.
 */
public abstract class AbstractExtensibleMessage
    implements
      ExtensibleMessage,
      MessageType,
      EmptyStringArray,
      Serializable
{

  /** The id of the message manager that created this chunk. */
  protected transient short messageManagerId;
  /** The number of reference to this message. */
  transient short referenceCounter = 0;
  
  /** Contains (name, chunk object) pairs. */
  protected Hashtable       chunks     = new Hashtable();
  /** Contains (name, chunk type) pairs. */
  protected Hashtable       chunkTypes = new Hashtable();

  /** The chunk names */
  protected String[]        chunkNames = null;

  // ---------------------------------------------------------------------------
  //  Implementation of the Message interface
  // ---------------------------------------------------------------------------

  /**
   * @see Message#getChunk(String)
   */
  public Object getChunk(String name)
  {
    return chunks.get(name);
  }

  /**
   * @see Message#getMessageType()
   */
  public MessageType getMessageType()
  {
    return this;
  }

  /**
   * @see Message#transfertChunkStates(Message)
   */
  public void transfertChunkStates(Message newInstance)
  {
    Iterator iter = chunks.entrySet().iterator();
    while (iter.hasNext())
    {
      Map.Entry e = (Map.Entry) iter.next();
      ((Chunk) e.getValue()).transfertState((Chunk) newInstance
          .getChunk((String) e.getKey()));
    }
  }

  /**
   * @see Message#getMessageManagerId()
   */
  public short getMessageManagerId()
  {
    return messageManagerId;
  }

  /**
   * @see Message#setMessageManagerId(short)
   */
  public void setMessageManagerId(short id)
  {
    messageManagerId = id;
  }

  /**
   * @see Message#recycle()
   */
  public void recycle()
  {
    if (!chunks.isEmpty())
    {
      System.err.println("WARNING : recycle not empty message");
      chunks.clear();
      chunkTypes.clear();
    }
  }

  // ---------------------------------------------------------------------------
  //  Implementation of the ExtensibleMessage interface
  // ---------------------------------------------------------------------------

  /**
   * @see ExtensibleMessage#addChunk(String, ChunkType, Object)
   */
  public void addChunk(String name, ChunkType chkType, Object chunk)
      throws ChunkAlreadyExistException
  {
    Object previousChunk = chunks.put(name, chunk);
    if (previousChunk != null)
    {
      chunks.put(name, previousChunk);
      throw new ChunkAlreadyExistException();
    }
    chunkTypes.put(name, chkType);
    chunkNames = null;
  }

  /**
   * @see ExtensibleMessage#removeChunk(String)
   */
  public Object removeChunk(String name)
  {
    if (chunkTypes.remove(name) != null)
    {
      chunkNames = null;
    }
    return chunks.remove(name);
  }

  //////////////////////////////////////////////////////////////////////////////
  //  Implementation of the MessageType interface
  //////////////////////////////////////////////////////////////////////////////

  /**
   * @see MessageType#getChunkNames()
   */
  public String[] getChunkNames()
  {
    if (chunkNames == null)
    {
      if (chunks.isEmpty())
      {
        chunkNames = EMPTY_STRING_ARRAY;
      }
      else
      {
        chunkNames = (String[]) chunks.keySet().toArray(EMPTY_STRING_ARRAY);
      }
    }
    return chunkNames;
  }

  /**
   * @see MessageType#getChunkNamesIterator()
   */
  public Iterator getChunkNamesIterator()
  {
    if (chunks.isEmpty())
    {
      return EmptyIterator.INSTANCE;
    }
    return chunks.keySet().iterator();
  }

  /**
   * @see MessageType#getChunkType(String)
   */
  public ChunkType getChunkType(String name)
  {
    return (ChunkType) chunkTypes.get(name);
  }

  /**
   * @see MessageType#isEmpty()
   */
  public boolean isEmpty()
  {
    return chunks.isEmpty();
  }

  // ---------------------------------------------------------------------------
  //  Public utility methods
  // ---------------------------------------------------------------------------

  /**
   * Increments the reference counter of this component.
   */
  public void incrementReferenceCounter()
  {
    referenceCounter++;
  }

  /**
   * Decrements the reference counter of this component.
   * 
   * @return <code>true</code> if the reference counter is null.
   */
  public boolean decrementReferenceCounter()
  {
    synchronized (this)
    {
      referenceCounter--;
      return (referenceCounter == 0);
    }
  }
}