/**
 * DREAM
 * Copyright (C) 2003 2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.message;

import java.util.Iterator;
import java.util.LinkedList;

/**
 * Implementation of extensible message, using a {@link java.util.LinkedList}to
 * store sub message
 */
public class BasicExtensibleMessage extends AbstractExtensibleMessage
{

  LinkedList    subMessages          = new LinkedList();
  Message[]     subMessagesArray     = null;
  MessageType[] subMessageTypesArray = null;

  /**
   * Constructor.
   * 
   * @param messageManagerId the id of the message manager that creates this
   *          message.
   */
  public BasicExtensibleMessage(short messageManagerId)
  {
    this.messageManagerId = messageManagerId;
  }

  /**
   * @see ExtensibleMessage#addSubMessage(Message)
   */
  public void addSubMessage(Message message)
  {
    if (subMessages.add(message))
    {
      subMessagesArray = null;
      subMessageTypesArray = null;
    }
  }

  /**
   * @see ExtensibleMessage#removeSubMessage(Message)
   */
  public boolean removeSubMessage(Message msg)
  {
    boolean removed = subMessages.remove(msg);
    if (removed)
    {
      subMessagesArray = null;
      subMessageTypesArray = null;
    }
    return removed;
  }

  /**
   * @see Message#getSubMessageIterator()
   */
  public Iterator getSubMessageIterator()
  {
    return subMessages.iterator();
  }

  /**
   * @see Message#getSubMessages()
   */
  public Message[] getSubMessages()
  {
    if (subMessagesArray == null)
    {
      if (subMessages.isEmpty())
      {
        subMessagesArray = EMPTY_MESSAGE_ARRAY;
      }
      else
      {
        subMessagesArray = (Message[]) subMessages.toArray(EMPTY_MESSAGE_ARRAY);
      }
    }
    return subMessagesArray;
  }

  /**
   * @see MessageType#getSubMessageTypes()
   */
  public MessageType[] getSubMessageTypes()
  {
    if (subMessageTypesArray == null)
    {
      if (subMessages.isEmpty())
      {
        subMessageTypesArray = EMPTY_MESSAGE_TYPE_ARRAY;
      }
      else
      {
        subMessageTypesArray = new MessageType[subMessages.size()];
        Iterator iter = getSubMessageIterator();
        int i = 0;
        while (iter.hasNext())
        {
          Message message = (Message) iter.next();
          subMessageTypesArray[i] = message.getMessageType();
          i++;
        }
      }
    }
    return subMessageTypesArray;
  }

  /**
   * @see MessageType#getSubMessageTypesIterator()
   */
  public Iterator getSubMessageTypesIterator()
  {
    return new SubMessageTypeIterator();
  }

  /**
   * @see MessageType#isSubTypeOf(MessageType)
   */
  public boolean isSubTypeOf(MessageType t)
  {
    // NOT IMPLEMENTED implement subtyping
    throw new UnsupportedOperationException("not yet implemented");
  }

  private class SubMessageTypeIterator implements Iterator
  {

    Iterator messageIterator = getSubMessageIterator();

    /**
     * @see java.util.Iterator#remove()
     */
    public void remove()
    {
      throw new UnsupportedOperationException();
    }

    /**
     * @see java.util.Iterator#hasNext()
     */
    public boolean hasNext()
    {
      return messageIterator.hasNext();
    }

    /**
     * @see java.util.Iterator#next()
     */
    public Object next()
    {
      return ((Message) messageIterator.next()).getMessageType();
    }

  }

}